//*** A P P L I C A T I O N   N O T E *************************************
//*
//* Title:               AVR-FPGA Interface Design #6
//* Version:             1.1
//* Last Updated:        October 8, 2001
//* Target:              AT94K05/10/40
//*
//* Support Hotline:     +1 (408) 436-4119
//* Support Email:       fpslic@atmel.com
//*
//*************************************************************************


//*************************************************************************
//*                              Include Files
//*************************************************************************

#include <ioat94k.h>
#include <macros.h>



//***************************************************************************
//*                        Global Variables & Defines
//***************************************************************************

#define sFINT0  0                           //FPGA Interrupt 0 Count SRAM Location
#define sFINT1  1                           //FPGA Interrupt 1 Count SRAM Location
#define sLSBMUL 2                           //Multiplication Result LSB SRAM Location
#define sMSBMUL 3                           //Multiplication Result MSB SRAM Location
#define sLSBRX  4                           //UART Received Byte LSB SRAM Location
#define sMSBRX  5                           //UART Received Byte MSB SRAM Location
#define sERRRX  6                           //UART Received Byte Error SRAM Location


unsigned char rCount0;                      //Count Register for FPGA Interrupt 0
unsigned char rCount1;                      //Count Register for FPGA Interrupt 1
unsigned char rError                        //Error Register for UART Functions


asm(".area memory(abs)\n"                   //The FPGA & AVR within FPSLIC can share
    ".org 0x0F9B\n"                         //the SRAM memory.  The AVR must "locate"
    "_DPSRAM:: .blkb 50");                  //data to be shared with the FPGA.  As of
                                            //October 2001, the only way to locate
                                            //variables in specific locations is to
                                            //use in-line assembly.

extern char DPSRAM[50];                     //Array of "Located" Dual Port SRAM



//***************************************************************************
//*                             Interrupt Vectors
//***************************************************************************

#pragma interrupt_handler isr_FPGA_INT0:FPGA_INT0
#pragma interrupt_handler isr_FPGA_INT1:FPGA_INT1


void isr_FPGA_INT0(void)
{
    unsigned int mResult;
    unsigned int rxResult;

    rCount0 = rCount0 + 1;                  //Increment Count of FPGA Interrupt 0 Calls
    DPSRAM[sFINT0] = rCount0;               //Store FPGA Interrupt Count @ sFINT0
    mResult = (rCount0 * rCount1);          //Multiply FPGA Interrupt Counts Together
    DPSRAM[sLSBMUL] = (char) mResult;       //Store LSB of Multiplication in SRAM
    DPSRAM[sMSBMUL] = (char) (mResult >> 8);//Store MSB of Multiplication in SRAM

    UDR0 = (char) mResult;                  //Transmit LSB of Multiplication
    UDR0 = (char) (mResult >> 8);           //Place MSB of Multiplication in Buffer

    while(!(UCSR1A & (1 << RXC1)));         //Wait Until RX Complete Bit Set
    DPSRAM[sLSBRX] = UDR1;                  //Store Received Byte in SRAM
    while(!(UCSR1A & (1 << RXC1)));         //Wait Until RX Complete Bit Set
    DPSRAM[sMSBRX] = UDR1;                  //Store Received Byte in SRAM

    rxResult  = (DPSRAM[sMSBRX] << 8);
    rxResult += DPSRAM[sLSBRX];

    if(rxResult != mResult)
        rError = rError + 1;

    PORTD = rCount0;                        //Output rCount0 on PORTD
    FISCR = ((0 << XFIS0) | (0 << XFIS1));  //Configure FPGA I/O Select Lines
    FISUA = rCount0;                        //Output Data to FPGA via I/O Select 0
}


void isr_FPGA_INT1(void)
{
    rCount1 = rCount1 + 1;                  //Increment Count of FPGA Interrupt 1 Calls
    DPSRAM[sFINT1] = rCount1;               //Store FPGA Interrupt Count @ sFINT1
    PORTE = rCount1;                        //Output rCount1 on PORTE
    FISCR = ((1 << XFIS0) | (0 << XFIS1));  //Configure FPGA I/O Select Lines
    FISUA = rCount1;                        //Output Data to FPGA via I/O Select 1
}



void initAT94K(void)
{
    CLI();                                  //Disable Global Interrupts

    rCount0 = 0;                            //Initialize rCount0 = $00
    rCount1 = 0;                            //Initialize rCount1 = $00
    rError  = 0;                            //Initialize rError  = $00

    FISCR = (1 << FIADR);                   //Enable FPGA Interrupt/Mask Access
    FISUA = ((1 << FINT0) | (1 << FINT1));  //Enable FPGA Interrupt 0 and Interrupt 1
    FISCR = (0 << FIADR);                   //Disable FPGA Interrupt/Mask Access

    UCSR0B = (1 << TXEN0);                  //Enable UART0 Transmit Feature
    UCSR1B = (1 << RXEN1);                  //Enable UART1 Receive Feature
    UBRR0  = 0x19;                          //Configure UART0 Baud Rate @ 9600 bps (4 MHz)
    UBRR1  = 0x19;                          //Configure UART1 Baud Rate @ 9600 bps (4 MHz)

    DDRD = 0xFF;                            //Configure PORTD as Output
    DDRE = 0xFF;                            //Configure PORTE as Output

    SEI();                                  //Enable Global Interrupts
}


void main(void)
{
    initAT94K();                            //Call initAT94K Function
    while(1);                               //Do Nothing, Wait for an Interrupt
}
